"""Define CLI interface for Dotenver."""

import argparse
import glob
import sys
from pathlib import Path

import colorama

from . import __version__, dotenver


def check_file_path(file_path):
    """Validate that the given file_path exists and can read."""
    file_path = Path(file_path)
    dotenv_path = dotenver.get_dotenv_path(file_path)

    if dotenv_path.name == file_path.name:
        print(colorama.Fore.RED, file=sys.stderr, end="")
        raise argparse.ArgumentTypeError(
            "'%s'. Template file can not be named '%s'" % (file_path, file_path.name)
        )

    try:
        open(file_path, "r")
    except FileNotFoundError:
        print(colorama.Fore.RED, file=sys.stderr, end="")
        raise argparse.ArgumentTypeError("'%s' could not be found" % file_path)
    except PermissionError:
        print(colorama.Fore.RED, file=sys.stderr, end="")
        raise argparse.ArgumentTypeError("'%s' is not readable" % file_path)

    try:
        open(dotenv_path, "a")
    except PermissionError:
        print(colorama.Fore.RED, file=sys.stderr, end="")
        raise argparse.ArgumentTypeError("'%s' is not writable" % dotenv_path)

    try:
        open(dotenv_path, "r")
    except FileNotFoundError:
        pass
    except PermissionError:
        print(colorama.Fore.RED, file=sys.stderr, end="")
        raise argparse.ArgumentTypeError("'%s' is not readable" % dotenv_path)

    return file_path


def cli():
    """Parse DotEnver templates and save to .env files."""
    colorama.init()

    parser = argparse.ArgumentParser(
        description="""Render DotEnver templates as .env files.
By default values in existing in .env files are respected, and missing variables are added.
"""
    )

    parser.add_argument(
        "-o", "--override", action="store_true", help="override current .env files."
    )

    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        "-r",
        "--recursive",
        action="store_true",
        help="process all template files recursively",
    )
    parser.add_argument(
        "--pattern",
        help=(
            "glob pattern used to search for templates. Only used with --recursive."
            " Default: '**/.env.example'"
        ),
        default="**/.env.example",
    )
    group.add_argument(
        "--version", action="store_true", help="print current DotEnver version"
    )
    group.add_argument(
        "files",
        metavar="file",
        type=check_file_path,
        nargs="*",
        default=False,
        help="path to file to process",
    )

    args = parser.parse_args()

    if args.version:
        print(__version__)
        return

    files = args.files
    if args.recursive:
        files = glob.glob(args.pattern, recursive=True)

        if not files:

            print(
                colorama.Fore.RED,
                "No template files found",
                file=sys.stderr,
                sep="",
            )
            return

        print(
            colorama.Fore.BLUE,
            "The following files will be parsed:",
            file=sys.stderr,
            sep="",
        )

        for file_ in files:
            print(colorama.Fore.BLUE, f" - {file_}", file=sys.stderr, sep="")

        print("", file=sys.stderr)
        confirmation = input(f'{colorama.Fore.YELLOW}Type "y" to confirm: ')
        print("", file=sys.stderr)

        if confirmation != "y":
            print(
                colorama.Fore.RED,
                "Will not proceed without your confirmation.",
                file=sys.stderr,
                sep="",
            )
            return

    dotenver.parse_files(files, override=args.override)
    return


if __name__ == "__main__":
    cli()



"""Generate .env files from .env.example templates."""

import io
import re
import sys
from pathlib import Path

import colorama
from faker import Faker
from jinja2 import Environment

VARIABLES = {}

VARIABLE_REGEX = r"""
    ^\s*
    (
        (?:export\s)?                   # Optional export command
        \s*
        ([^\s=#]+)                      # Variable name
    )
    (?:
        \s*=\s*?                        # Assignment
        (.*?)                           # Verbatim value, no parsing done
        {0}                             # Capture placeholder
    )?
    \s*$
"""

VALUES_REGEX = re.compile(VARIABLE_REGEX.format(""), re.VERBOSE)

TEMPLATE_REGEX = re.compile(
    VARIABLE_REGEX.format(
        r"""
        (?:
            \#\#\ +dotenver:            # Start of the dotenver comment
            (?:
                ([^\(\s#:]+)             # Faker generator to use
                (?::
                    ([^\(\s#]+)         # Value name
                )?
            )
            (?:\((
                .*                      # Arguments to pass to the generator
            )\))?
        )?                              # dotenver comment is optional
    """
    ),
    re.VERBOSE,
)
FAKE = Faker()


def get_value_key(generator, name):
    """
    Return a key for the given generator and name pair.

    If name None, no key is generated.
    """
    if name is not None:
        return f"{generator}+{name}"

    return None


def dotenver(generator, name=None, quotes=None, escape_with="\\", **kwargs):
    r"""
    Generate fake data from the given `generator`.

    If a `name` is given, the value from the given `generator` and `name`
    will be saved and used for subsequent calls.
    In those cases only the quotes argument is honored for each call.

    The returned value will be optionally surrounded with single or
    double quotes as specified by `quotes`, and escaped with `escape_with`,
    which is a backslash `\` by default.
    """
    if quotes not in [None, "'", '"']:
        raise ValueError("quotes must be a single `'` or double `\"` quote")

    key = get_value_key(generator, name)

    value = str(VARIABLES.get(key, getattr(FAKE, generator)(**kwargs)))

    if key and key not in VARIABLES:
        VARIABLES[key] = value

    if quotes:
        value = value.replace(quotes, f"{escape_with}{quotes}")
        value = f"{quotes}{value}{quotes}"

    return value


def parse_stream(template_stream, current_dotenv):
    """Parse a dotenver template."""
    jinja2_template = io.StringIO()

    env = Environment(keep_trailing_newline=True)
    env.globals["dotenver"] = dotenver

    extra_variables = current_dotenv.copy()

    for line in template_stream:
        match = TEMPLATE_REGEX.match(line)
        if match:
            left_side, variable, value, generator, name, arguments = match.groups()

            if variable in current_dotenv:
                current_value = current_dotenv[variable][1]
                try:
                    del extra_variables[variable]
                except KeyError:
                    pass

                # Keep track of existing named values.
                key = get_value_key(generator, name)
                if key:
                    try:
                        VARIABLES[key]
                    except KeyError:
                        VARIABLES[key] = current_value

                line = (
                    f"{left_side}={current_value}"
                    if current_value is not None
                    else left_side
                )
            elif generator:
                dotenver_args = f"'{generator}'"
                if name:
                    dotenver_args = f"{dotenver_args}, '{name}'"
                if arguments:
                    dotenver_args = f"{dotenver_args}, {arguments}"
                line = f"{left_side}={{{{ dotenver({dotenver_args}) }}}}"
            elif value:
                line = f"{left_side}={value}"
            else:
                line = left_side

        jinja2_template.write(f"{line.strip()}\n")

    if extra_variables:
        jinja2_template.write(
            """
######################################
# Variables not in Dotenver template #
######################################

"""
        )
        for left_side, value in extra_variables.values():
            template_string = f"{left_side}={value}" if value is not None else left_side
            jinja2_template.write(f"{template_string}\n")

    template = env.from_string(jinja2_template.getvalue())

    return template


def get_dotenv_path(template_path):
    """Return the .env path for the given template path."""
    if template_path.suffix == ".example":
        return template_path.with_suffix("")

    return template_path.with_name(".env")


def get_dotenv_dict(dotenv_path):
    """
    Read a .env file and return a dictionary of the parsed data.

    Each item has the VARIABLE as the key, and the value is a tuple:
    (assignment, value)

    If the file does not exist, return an empty dict.
    """
    values = dict()
    try:
        with open(dotenv_path, "r") as dotenv_file:
            for line in dotenv_file:
                match = VALUES_REGEX.match(line)
                if match:
                    assignment, variable, value = match.groups()
                    values[variable] = (assignment, value)
    except FileNotFoundError:
        pass
    except Exception:
        print(
            colorama.Fore.RED,
            f"The following exception ocurred while reading '{dotenv_path}'",
            colorama.Fore.YELLOW,
            sep="",
            file=sys.stderr,
        )
        raise
    return values


def parse_files(templates_paths, override=False):
    """Parse multiple dotenver templates and generate or update a .env for each."""
    colorama.init()
    jinja2_templates = {}
    rendered_templates = {}

    # First pass will:
    # - capture all variables form templates and .env files
    # - capture existing values from .env files
    # - generate Jinja2 template
    for _template_path in templates_paths:
        template_path = Path(_template_path)
        current_env = (
            get_dotenv_dict(get_dotenv_path(template_path)) if not override else {}
        )
        try:
            with open(template_path, "r") as template_file:
                jinja2_templates[template_path] = parse_stream(
                    template_file, current_env
                )
        except Exception:
            print(
                colorama.Fore.RED,
                f"The following exception ocurred while processing template"
                f" '{template_path}'",
                colorama.Fore.YELLOW,
                sep="",
                file=sys.stderr,
            )
            raise

    # Second pass renders the templates.
    # Rendering on a second pass ensures all named values from .env files
    # were captured, and can be assigned to named dotenvers in templates.
    for template_path, jinja2_template in jinja2_templates.items():
        try:
            rendered_templates[template_path] = jinja2_template.render()
        except Exception:
            print(
                colorama.Fore.RED,
                f"The following exception ocurred while processing template"
                f" '{template_path}'",
                colorama.Fore.YELLOW,
                sep="",
                file=sys.stderr,
            )
            raise

    for template_path, rendered_template in rendered_templates.items():
        dotenv_path = get_dotenv_path(template_path)
        try:
            with open(dotenv_path, "w") as dotenv_file:
                dotenv_file.write(rendered_template)
        except Exception:
            print(
                colorama.Fore.RED,
                f"The following exception ocurred while writing to '{dotenv_path}'",
                colorama.Fore.YELLOW,
                sep="",
                file=sys.stderr,
            )
            raise

        print(
            colorama.Fore.GREEN,
            f"'{template_path}' rendered to '{dotenv_path}'",
            sep="",
            file=sys.stderr,
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dotenver']

package_data = \
{'': ['*']}

install_requires = \
['colorama>=0.4.1,<0.5.0', 'faker>=1.0,<2.0', 'jinja2>=2.10,<3.0']

extras_require = \
{'testing': ['pytest>=5.3.2,<6.0.0',
             'docutils>=0.16,<0.17',
             'pygments>=2.5.2,<3.0.0',
             'toml>=0.10.0,<0.11.0']}

entry_points = \
{'console_scripts': ['dotenver = dotenver.cli:cli']}

setup_kwargs = {
    'name': 'dotenver',
    'version': '1.2.1',
    'description': 'Automatically generate .env files from .env.example template files',
    'long_description': '============================\nPython DotEnver\n============================\n\n.. image:: https://badge.fury.io/py/dotenver.svg\n    :target: https://badge.fury.io/py/dotenver\n\n.. image:: https://travis-ci.org/jmfederico/dotenver.svg?branch=master\n    :target: https://travis-ci.org/jmfederico/dotenver\n\n.. image:: https://img.shields.io/badge/code%20style-black-000000.svg\n    :target: https://github.com/ambv/black\n\nA Python app to generate dotenv (.env) files from templates.\n\n\nFeatures\n--------\n\n* Automatic .env file generation from .env.example files\n* Useful for CI or Docker deployments\n* Uses Jinja2_ as rendering engine\n* Uses Faker_ for value generation\n\n\nQuickstart\n----------\n\n1. Install **Python DotEnver**\n\n   .. code-block:: console\n\n    $ pip install dotenver\n\n2. Create a **.env.example** following this example\n\n   .. code-block:: ini\n\n    # Full line comments will be kept\n\n    # Simple usage\n    NAME= ## dotenver:first_name\n\n    # Pass parameters to fakers\n    ENABLED= ## dotenver:boolean(chance_of_getting_true=50)\n\n    # Name your values\n    MYSQL_PASSWORD= ## dotenver:password:my_password(length=20)\n    # And get the same value again, when the name is repeated.\n    DB_PASSWORD= ## dotenver:password:my_password()\n\n    # Output your values within double or single quotes\n    DOUBLE_QUOTED= ## dotenver:last_name(quotes=\'"\')\n    SINGLE_QUOTED= ## dotenver:last_name(quotes="\'")\n\n    # Literal values are possible\n    STATIC_VARIABLE=static value\n\n    # export syntax can be used\n    export EXPORTED_VARIABLE=exported\n\n3. Run python **DotEnver** form the CLI\n\n   .. code-block:: console\n\n    $ dotenver -r\n\n4. You now have a new **.env** file ready to use.\n\n5. For more usage options run\n\n   .. code-block:: console\n\n    $ dotenver -h\n\n\nDocker\n------\n\nA Docker image `is provided <Dotenver image_>`_. To use it, mount your source code to\n`/var/lib/dotenver/` and run the container.\n\n.. code-block:: console\n\n    $ docker run -ti --rm -v "${PWD}:/var/lib/dotenver/" jmfederico/dotenver\n\n.. _Faker: https://faker.readthedocs.io\n.. _Jinja2: http://jinja.pocoo.org\n.. _jmfederico: https://github.com/jmfederico\n.. _`Dotenver image`: https://hub.docker.com/r/jmfederico/dotenver\n',
    'author': 'Federico Jaramillo Martínez',
    'author_email': 'federicojaramillom@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jmfederico/dotenver',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
